@@ -21,16 +21,25 @@
     setNames(files, sub(".txt$", "", basename(files)))
 }
 
-.getDataInFile <-
-    function(name)
+.normalize_organism <- function(organism)
 {
-    ##name will always be informat Homo sapiens
-    ## files are in format of : Homo_sapiens.txt
-    filename <- paste0(.getDatadir(),"/",sub(" ", "_", name),".txt")
+    parts <- CharacterList(strsplit(organism, "_| "))
+    parts_eltNROWS <- elementNROWS(parts)
+    ## If 3 parts or more (e.g. "Canis_lupus_familiaris") then remove part 2.
+    idx3 <- which(parts_eltNROWS >= 3L)
+    if (length(idx3) != 0L)
+        parts[idx3] <- parts[idx3][rep.int(list(-2L), length(idx3))]
+    unstrsplit(parts, sep="_")
+}
+
+.getDataInFile <- function(organism)
+{
+    organism2 <- .normalize_organism(organism)
+    filename <- paste0(.getDatadir(), "/", organism2, ".txt")
     if (file.exists(filename)) {
         read.table(filename, header=TRUE, sep="\t", stringsAsFactors=FALSE)
     } else {
-        stop("The species, " ,name, " is not supported by GenomeInfoDb")
+        stop("Organism ", organism, " is not supported by GenomeInfoDb")
     }
 
 }
@@ -52,28 +61,15 @@
 
 
 .isSupportedSeqnamesStyle <-
-    function(species, style)
+    function(organism, style)
 {
-    if (missing(species) || missing(style))
-        stop("'species' or 'style' missing")
-
-    species <- sub(" ", "_", species)
+    organism <- .normalize_organism(organism)
     possible <- lapply(.getNamedFiles(), scan, nlines=1, what=character(),
                        quiet=TRUE)
-    availStyles <- possible[[species]]
+    availStyles <- possible[[organism]]
     style %in% availStyles[-which(availStyles %in% c("circular","auto","sex"))]
 }
 
-.isSupportedSeqnames <-
-    function(species, style, seqnames)
-{
-    if (missing(species) || missing(style) || missing(seqnames))
-        stop("'species', 'style' or 'seqnames' missing")
-
-    trueSeq <- extractSeqlevels(species=species,style=style)
-    all(trueSeq %in% seqnames)
-}
-
 .supportedSeqnameMappings <-
     function()
 {
@@ -95,9 +91,9 @@
     }else{
         ##vec is in format "Homo_sapiens.UCSC"
         vec <- names(which(unlistgot2==max(unlistgot2)))
-        species <- sub("_"," ",sub("(.*?)[.].*", "\\1", vec))
+        organism <- .normalize_organism(sub("(.*?)[.].*", "\\1", vec))
         style <- gsub("^[^.]+.","", vec)
-        ans <- list(species=species, style=style) 
+        ans <- list(species=organism, style=style) 
     }
     ans
 }
@@ -123,13 +119,13 @@ setMethod("seqlevelsStyle", "character",
     
     seqnames <- unique(x)      
     ans <- .guessSpeciesStyle(seqnames)
-    
+
     ## 3 cases -
     ## 1. if no style found - ans is na - stop with message 
     ## 2. if multiple styles returned then print message saying that it could be 
     ## any of these styles
     ## 3. if one style returned - hurray!
-    
+
     if(length(ans)==1){
         if(is.na(ans)){
             txt <- "The style does not have a compatible entry for the
@@ -138,14 +134,7 @@ setMethod("seqlevelsStyle", "character",
             stop(paste(strwrap(txt, exdent=2), collapse="\n"))
         }
     }
-    
-    
-    style <- unique(ans$style)
-    
-    if(length(style)>1)
-        message("warning! Multiple seqlevels styles found.")
-      
-    style
+    unique(ans$style)
 })
 
 ### The default methods work on any object 'x' with working "seqlevels"
@@ -153,32 +142,51 @@ setMethod("seqlevelsStyle", "character",
 
 setMethod("seqlevelsStyle", "ANY", function(x) seqlevelsStyle(seqlevels(x)))
 
-setReplaceMethod("seqlevelsStyle", "ANY",
-    function(x, value)
-    {
-        x_seqlevels <- seqlevels(x)
-        renaming_maps <- mapSeqlevels(x_seqlevels, value, drop=FALSE)
-        if (nrow(renaming_maps) == 0L) {
-            msg <- c("found no sequence renaming map compatible ",
-                     "with seqname style \"", value, "\" for this object")
-            stop(msg)
-        }
-        ## Use 1st best renaming map.
-        if (nrow(renaming_maps) != 1L) {
-            msg <- c("found more than one best sequence renaming map ",
-                     "compatible with seqname style \"", value, "\" for ",
-                     "this object, using the first one")
-            warning(msg)
-            renaming_maps <- renaming_maps[1L, , drop=FALSE]
-        }
-        new_seqlevels <- as.vector(renaming_maps)
-        na_idx <- which(is.na(new_seqlevels))
-        new_seqlevels[na_idx] <- x_seqlevels[na_idx]
-        seqlevels(x) <- new_seqlevels
-        x
+.replace_seqlevels_style <- function(x_seqlevels, value)
+{
+    renaming_maps <- mapSeqlevels(x_seqlevels, value, drop=FALSE)
+    if (nrow(renaming_maps) == 0L) {
+        msg <- c("found no sequence renaming map compatible ",
+                 "with seqname style \"", value, "\" for this object")
+        stop(msg)
     }
+    ## Use 1st best renaming map.
+    if (nrow(renaming_maps) != 1L) {
+        msg <- c("found more than one best sequence renaming map ",
+                 "compatible with seqname style \"", value, "\" for ",
+                 "this object, using the first one")
+        warning(msg)
+        renaming_maps <- renaming_maps[1L, , drop=FALSE]
+    }
+    new_seqlevels <- as.vector(renaming_maps)
+    na_idx <- which(is.na(new_seqlevels))
+    new_seqlevels[na_idx] <- x_seqlevels[na_idx]
+    new_seqlevels
+}
+
+setReplaceMethod("seqlevelsStyle", "character",
+    function (x, value)
+    {
+        x_seqlevels <- unique(x)
+        if (!(is.character(value) && length(value) >= 1L))
+            stop("the supplied seqlevels style must be a single string")
+        if (length(value) > 1L) {
+            warning(wmsg("more than one seqlevels style supplied, ",
+                         "using the 1st one only"))
+            value <- value[[1L]]
+        }
+        new_seqlevels <- .replace_seqlevels_style(x_seqlevels, value)
+        new_seqlevels[match(x, x_seqlevels)]
+     }
 )
 
+setReplaceMethod("seqlevelsStyle", "ANY",
+     function (x, value)
+     {
+         seqlevelsStyle(seqlevels(x)) <- value 
+         x
+     }
+)
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Related low-level utilities
@@ -242,7 +250,7 @@ mapSeqlevels <-
     if (!is.character(seqnames))
         stop("'seqnames' must be a character vector")
     if (!isSingleString(style))
-        stop("the supplied seqname style must be a single string")
+        stop("the supplied seqlevels style must be a single string")
     if (!.isTRUEorFALSE(best.only))
         stop("'best.only' must be TRUE or FALSE")
     if (!.isTRUEorFALSE(drop))
